import tempfile

from google.cloud import firestore, secretmanager
import json
import requests
import os
import uuid
import re

ONESIGNAL_APP_ID = os.getenv('ONESIGNAL_APP_ID', "")
PROJECT_ID = os.getenv('PROJECT_ID', "424159745652")
ENV = os.environ.get("ENV", "prod")


def generate_device_token():
    try:
        url = "https://onesignal.com/api/v1/players"

        payload = {
            "device_type": 5,
            "language": "en",
            "timezone": "-28800",
            "game_version": "1.1.1",
            "device_model": "iPhone5,1",
            "device_os": "15.1.1",
            "session_count": 600,
            "tags": {
                "first_name": "Jon",
                "last_name": "Smith",
                "level": "99",
                "amount_spent": "6000",
                "account_type": "VIP"
            },
            "amount_spent": "100.99",
            "playtime": 600,
            "notification_types": 1,
            "lat": 37.563,
            "long": 122.3255,
            "country": "US",
            "timezone_id": "Asia/Kolkata",
            "app_id": ONESIGNAL_APP_ID
        }
        headers = {
            "accept": "application/json",
            "Content-Type": "application/json"
        }

        response = requests.post(url, json=payload, headers=headers)
        return response.json()
    except Exception as e:
        print(e)


def generate_notification_id():
    return str(uuid.uuid4())


def validate_email(email):
    """Validates an email address using a regular expression."""
    email_regex = r'^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'
    return re.match(email_regex, email) is not None


def validate_phone_number(phone_number):
    """Validates a phone number."""
    phone_regex = r"^\+(?:[0-9] ?){6,14}[0-9]$"
    return re.match(phone_regex, phone_number) is not None


def sanitize_topic_name(topic_name):
    # Replace "@" with "-" to make the topic name valid
    sanitized_topic_name = topic_name.replace('@', '-')
    return sanitized_topic_name


def get_db_via_firebase_credentials():
    try:
        if ENV == "dev":
            return firestore.Client.from_service_account_json("firebase_credentials.json")
        elif ENV == "gcp":
            client = secretmanager.SecretManagerServiceClient()
            secret_name_path = f"projects/{PROJECT_ID}/secrets/ENV_FILE/versions/latest"
            response = client.access_secret_version(name=secret_name_path)
            secret_data = response.payload.data.decode('UTF-8')
            secret_json = json.loads(secret_data)
            return secret_json
        else:
            client = firestore.Client()
            return client
    except Exception as e:
        print(f"Error getting firebase credentials: {e}")
        return None


def get_secret(secret_name, version="latest"):
    client = secretmanager.SecretManagerServiceClient()
    secret_name_path = f"projects/{PROJECT_ID}/secrets/{secret_name}/versions/{version}"
    response = client.access_secret_version(name=secret_name_path)
    return response.payload.data.decode('UTF-8')


ENV_VALS = {}


def set_secret():
    try:
        response = secretmanager.SecretManagerServiceClient().access_secret_version(name=f"projects/{PROJECT_ID}/secrets/ENV_FILE/versions/latest")
        secret_data = response.payload.data.decode('UTF-8')
        secret_json = json.loads(secret_data)
        if secret_json and len(secret_json) > 0:
            ENV_VALS = secret_json
            print("Loaded environment variables", ENV_VALS)
        else:
            print("No environment variables loaded")
    except Exception as e:
        print(f"ERROR while setting secret: {e}")


def get_env_secret(key):
    try:
        return ENV_VALS.get(key)
    except Exception as e:
        print(f"Error getting ENV_FILE secrets: {e}")
        return None


def get_db():
    try:
        creds_json = get_secret("FIREBASE_CREDENTIALS")
        db_secret_json = json.loads(creds_json)
        with tempfile.NamedTemporaryFile(delete=False, mode='w', encoding='utf-8') as temp_file:
            json.dump(db_secret_json, temp_file, indent=4)
            temp_file_path = temp_file.name

        db = firestore.Client.from_service_account_json(temp_file_path)
        return db
    except Exception as e:
        print(f"Error getting DB: {e}")
        db = firestore.Client()
        return db



import os
from passlib.context import CryptContext
from pingpongx.models import SignupRequest, LoginRequest
from pingpongx.services.firestore_service import db
import datetime
import jwt
from pingpongx.utils import validate_phone_number, validate_email, get_secret

pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")
JWT_SECRET = get_secret("JWT_SECRET")
JWT_ALGORITHM = get_secret("JWT_ALGORITHM")


def generate_jwt(username: str):
    """Generate a JWT token with the username."""
    try:
        payload = {
            "username": username,
            "exp": datetime.datetime.utcnow() + datetime.timedelta(hours=1),  # Token expires in 1 hour
            "iat": datetime.datetime.utcnow()
        }
        return jwt.encode(payload, JWT_SECRET, algorithm=JWT_ALGORITHM)
    except Exception as e:
        print(e)
        return None


def hash_password(password: str) -> str:
    """Hash the password using bcrypt."""
    return pwd_context.hash(password)


def verify_password(plain_password: str, hashed_password: str) -> bool:
    """Verify a plain password against the hashed one."""
    return pwd_context.verify(plain_password, hashed_password)


def register_user(username: str, password: str):
    """Register a new user."""
    try:
        username = username.strip().lower()
        if "@" in username:
            valid_username = validate_email(username)
        else:
            valid_username = validate_phone_number(username)

        if username == "" or valid_username is None:
            return {"message": "Invalid credentials", "success": False}
        user_ref = db.collection("users").document(username)
        if user_ref.get().exists:
            return {"message": "User already exists", "success": False}

        hashed_password = hash_password(password)
        user_ref.set({
            "username": username,
            "password": hashed_password,
            "created_at": datetime.datetime.utcnow().isoformat()
        })
        return {"message": "User registered successfully", "success": True}
    except Exception as e:
        return {"message": str(e), "success": False}


def login_user(username: str, password: str):
    """Login user by verifying credentials."""
    try:
        username = username.strip().lower()
        if "@" in username:
            valid_username = validate_email(username)
        else:
            valid_username = validate_phone_number(username)

        if username == "" or valid_username is None:
            return {"message": "Username can't be empty", "success": False}

        user_ref = db.collection("users").document(username)
        user_data = user_ref.get()
        if not user_data.exists:
            return {"message": "Invalid credentials", "success": False}

        user_data = user_data.to_dict()
        if not verify_password(password, user_data["password"]):
            return {"message": "Invalid credentials", "success": False}

        token = generate_jwt(username)
        if token:
            return {"message": "Login successful", "username": username, "token": token, "success": True}
        return {"message": "Login failed!", "success": False}
    except Exception as e:
        return {"message": str(e), "success": False}


def signup(request: SignupRequest):
    """Signup API."""
    return register_user(request.username, request.password)


def login(request: LoginRequest):
    """Login API."""
    return login_user(request.username, request.password)


def sender_record(sender: str, channel: str = None):
    try:
        if channel is None or channel not in ["email", "sms"]:
            return {"message": "Invalid channel", "success": False}

        doc_key = sender + "_" + channel
        user_ref = db.collection("sender_record").document(doc_key)
        user_data = user_ref.get()
        count = 1
        current_time = datetime.datetime.utcnow()

        if user_data.exists:
            user_data = user_data.to_dict()
            last_updated = user_data.get("last_updated", None)

            if last_updated:
                last_updated_time = datetime.datetime.fromisoformat(last_updated)
                time_difference = abs(current_time - last_updated_time)
                if time_difference.total_seconds() > 48 * 3600:  # 48 hours in seconds
                    count = 1  # Reset count if more than 48 hours
                else:
                    count = user_data["count"] + 1
            else:
                count = user_data["count"] + 1

            if channel == "email" and count < 5 or channel == "sms" and count < 3:
                user_ref.set({
                    "username": sender,
                    "channel": channel,
                    "count": count,
                    "last_updated": datetime.datetime.utcnow().isoformat() if count == 1 else last_updated
                })
                return {"message": "done", "success": True}
            else:
                return {"message": "unauthorized", "success": False}
        else:
            user_ref.set({
                "username": sender,
                "channel": channel,
                "count": count,
                "last_updated": datetime.datetime.utcnow().isoformat()
            })
            return {"message": "done", "success": True}

    except Exception as e:
        return {"message": str(e), "success": False}


def get_sender_record(sender: str, channel: str = None):
    try:
        if channel is None or channel not in ["email", "sms"]:
            return None

        current_time = datetime.datetime.utcnow()
        doc_key = sender + "_" + channel
        user_ref = db.collection("sender_record").document(doc_key)
        user_data = user_ref.get()
        if user_data.exists:
            user_data = user_data.to_dict()
            last_updated = user_data.get("last_updated", None)

            if last_updated:
                last_updated_time = datetime.datetime.fromisoformat(last_updated)
                time_difference = abs(current_time - last_updated_time)
                if time_difference.total_seconds() > 48 * 3600:  # 48 hours in seconds
                    count = 1   # Reset count if more than 48 hours
                else:
                    count = user_data["count"]
            else:
                count = user_data["count"]
            return count
        return 1

    except Exception as e:
        print("get_sender_record: ", e)
        return None

